/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#ifndef CBROADCASTSERVER_H
#define CBROADCASTSERVER_H

#include <QHostAddress>
#include <QMap>
#include <QTimer>
#include <QSystemTrayIcon>
#include <QUdpSocket>

#include <discovery/CMachine.h>

class CBroadcastServer : public QUdpSocket
{
    Q_OBJECT

    public:

        CBroadcastServer();

        void Init();

        MachineMap GetMachineMap() { return m_machines; }
        bool AnyMachinesOnline() { return m_machines.size(); }
        bool MachineExists(QString id) { return m_machines.contains(id); }
        CMachine GetMachine(QString id) { return m_machines[id]; }

    signals:

        // TODO: remove these
        void Error(QString);
        void Information(QString);

    private slots:

        void OnMessage();
        void OnTimer();

    private:

        void UpdateMachineEntry(QHostAddress, QString, QByteArray, quint16, qint64);
        void CheckForExpiredMachines();

        QTimer       m_timer;
        QHostAddress m_broadcast_address;
        qint64       m_start_time;

        MachineMap m_machines;
};

#endif // CBROADCASTSERVER_H
